import subprocess
import logging
import re


logger = logging.getLogger('netd')


def check_output(command, *args, **kwargs):
    '''
    Basically a facade over the check_output
    command that we can do logging with.
    '''
    messages = kwargs.pop('messages', None)
    verbose = kwargs.pop('verbose', False)
    fmt_command = command
    if isinstance(fmt_command, (list, tuple)):
        fmt_command = ' '.join(fmt_command)
    try:
        result = subprocess.check_output(command, *args, **kwargs)
        # success means return code 0
        logger.debug('Ran command: {}, Return code: {}'.format(
            fmt_command, 0))

        if verbose and messages is not None:
            if "output" not in messages:
                messages["output"] = ''
            if result:
                messages["output"] += "{0}\n---------\n{1}\n\n".format(
                    fmt_command, result)
            else:
                messages["output"] += "Ran {0}.\n\n".format(fmt_command)

        return result
    except subprocess.CalledProcessError as exc:
        logger.debug('Ran command: {}, Return code: {}, Output: {}'.format(
            fmt_command, exc.returncode, exc.output))
        if verbose and messages:
            if "output" not in messages:
                messages["output"] = ''
            if exc.output:
                messages["output"] += "{0}\n---------\n{1}\n\n".format(
                    fmt_command, exc.output)
            else:
                messages["output"] += "Ran {0}. Exited {1}\n\n".format(
                    fmt_command, exc.returncode)

        # just bubble, we're here for the logging
        raise


def safe_unidecode(val):
    '''
    Convert a string in a safe way to unicode
    >>> safe_unidecode(b'foobar')
    'foobar'
    :param val: str to convert
    '''
    if isinstance(val, unicode):
        # already decoded
        return val

    for codec in ('utf-8', 'windows-1252', 'latin-1'):
        try:
            return val.decode(codec)
        except UnicodeDecodeError:
            pass
    return val.decode('utf-8', errors='replace')


class Config:
    def __init__(self, config_file):
        self.config_file = config_file
        self.show_linux_command = None
        self.color_diffs = None
        self.blacklist = None
        self.components = None


    def parse(self):
        try:
            # Read the configuration file.
            with open(self.config_file) as f:
                self.config_text = f.read()
        except OSError as e:
            exit("Couldn't read the configuration file, {0}.  ({1})".format(self.config_file, e))

        # These boolean attributes pertain to how the output returned to the user should look.
        self.show_linux_command = re.search(r"""^\s*show_linux_command\s*=\s*True""", self.config_text, re.IGNORECASE | re.MULTILINE) is not None
        self.color_diffs = re.search(r"""^\s*color_diffs\s*=\s*True""", self.config_text, re.IGNORECASE | re.MULTILINE) is not None

        # Populate the permissions attributes.  These are four frozensets that specify who can do what.  If the permission is
        # not specified in the configuration file, or if it is set to blank, the resulting attribute will be an empty frozenset.
        for permissions_group in ("users_with_edit", "users_with_show", "groups_with_edit", "groups_with_show"):
            users_re = re.search(r"""^\s*{0}\s*=\s*(?P<users>\S[\S\t ,]*)$""".format(permissions_group), self.config_text, re.IGNORECASE | re.MULTILINE)
            # TODO - Should users be mapped to lowercase?  Are "root" and "ROOT" different to the OS?
            setattr(self, permissions_group, frozenset([] if (users_re is None) else users_re.group("users").replace(',', ' ').split()))

        # Populate the blacklist for ifupdown.  Blacklisted terms serve to filter the available ifupdown2 commands.
        blacklist_re = re.search(r"""^\s*ifupdown_blacklist\s*=\s*(?P<blacklist>\S[\S\t ,]*)$""", self.config_text, re.IGNORECASE | re.MULTILINE)
        self.blacklist = frozenset([] if (blacklist_re is None) else blacklist_re.group("blacklist").replace(',', ' ').split())

        # Get a lowercase set of all components configured to be enabled.  The netmisc and ports components are always available.
        self.enabled_wrappers = set(["netmisc"])
        self.enabled_plugins = set(["ports"])
        self.components = map(str.lower, re.findall(r"""^\s*enable_(\S+)\s*=\s*True""", self.config_text, re.IGNORECASE | re.MULTILINE))
